#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "urarlib.h"
#include "RARFile.h"

JNIEXPORT void JNICALL Java_RARFile_loadList(JNIEnv *env, jobject obj, jstring rarfile)
{
	ArchiveList_struct *list = NULL;

	jstring str;
	jclass cls = (*env)->GetObjectClass(env, obj);
	jmethodID mid = (*env)->GetMethodID(env, cls, "addFile", "(Ljava/lang/String;JJJJ)V");
	
	const char *filename = (*env)->GetStringUTFChars(env, rarfile, 0);

	printf("\nInitialized...\n");

	if (mid == 0) 
	{
		printf("Could not find method!");
        return;
    }

	printf("Loading %s...\n", filename);

    urarlib_list((char *)filename, (ArchiveList_struct*)&list);

	while(list != NULL)
	{
		str = (*env)->NewStringUTF(env, list->item.Name);

		(*env)->CallVoidMethod(env, obj, mid, str, (jlong)list->item.PackSize, (jlong)list->item.UnpSize, (jlong)list->item.FileTime, (jlong)list->item.FileAttr);

		(ArchiveList_struct*)list = (ArchiveList_struct*)list->next; 
	}

    (*env)->ReleaseStringUTFChars(env, rarfile, filename);
}


JNIEXPORT jbyteArray JNICALL Java_RARFile_getData(JNIEnv *env, jobject obj, jstring file, jstring arc, jstring pass)
{
	char *data;
	unsigned long size = 0;
	unsigned long i = 0;

	jbyteArray array;
	jbyte *body;

	const char *filename = (*env)->GetStringUTFChars(env, file, 0);
	const char *arcfile  = (*env)->GetStringUTFChars(env, arc , 0);
	const char *password = (*env)->GetStringUTFChars(env, pass, 0);

	if(urarlib_get(&data, &size, (char *)arcfile, (char *)filename, (char *)password))
    { 
		printf("OK!\n"); 
    } 
	else 
    { 
		printf("Error! Could not open %s from %s!\n", arcfile, filename); 
    } 

	(*env)->ReleaseStringUTFChars(env, file, filename);
	(*env)->ReleaseStringUTFChars(env, arc , arcfile);
	(*env)->ReleaseStringUTFChars(env, pass, password);

	
	array = (*env)->NewByteArray(env, (jsize)size);
	body = (*env)->GetByteArrayElements(env, array, 0);

	for(i=0; i<size; i++)
	{
		body[i] = data[i];
	}

	(*env)->ReleaseByteArrayElements(env, array, body, 0);

	return array;
}

