/*
 * BufferInputStream.java
 *
 * Created on October 6, 2001, 11:40 PM
 */

package util;

import java.io.*;

/**
 *
 * @author  jbanes
 * @version 
 */
public class BufferInputStream extends InputStream
{
    private byte[] buffer;
    int pos = 0;
    int mark = -1;
    int readlimit = 0;
    
    public BufferInputStream(InputStream in) throws IOException
    {
        boolean complete = false;
        
        byte[] buffer = new byte[100];
        this.buffer = new byte[buffer.length];
        
        while(!complete)
        {    
            int length = in.read(buffer);
            complete = (length < buffer.length);
            
            for(int i=0; i<length; i++)
            {
                this.buffer[i+pos] = buffer[i];
            }
            
            pos += length;
            
            if(!complete) if(pos >= buffer.length) resize(this.buffer.length+buffer.length);
        }
        
        resize(pos);
        pos = 0;
        in.close();
    }
    
    public BufferInputStream(InputStream in, int length) throws IOException
    {
        buffer = new byte[length];
        in.read(buffer);
        in.close();
    }
    
    public BufferInputStream(byte[] buffer)
    {
        this.buffer = buffer;
    }

    private void resize(int length)
    {
        byte[] old = buffer;
        
        buffer = new byte[length];
        
        for(int i=0; i<pos; i++)
        {
            buffer[i] = old[i];
        }
    }
    
    public int available() throws IOException
    {
        return buffer.length-pos;
    }
    
    public void close() throws IOException
    {
        pos = buffer.length;
    }
    
    public synchronized void mark(int readlimit) 
    {
        mark = pos;
        this.readlimit = readlimit;
    }
    
    public boolean markSupported()
    {
        return true;
    }
    
    public int read() throws IOException
    {
        int clear = 0xFF;
        
        if(pos >= buffer.length) return -1;
        
        int ret = clear & buffer[pos];
        
        if(ret < 0) ret |= clear;
        pos++;
      
        return ret;
    }
    
    public int read(byte[] b) throws IOException
    {
        return read(b, 0, b.length);
    }
    
    public int read(byte[] b, int off, int len) throws IOException
    {
        int length = Math.min(len, available());
        int actual = 0;
        
        for(int i=0; i<length; i++)
        {
            b[off+i] = buffer[pos];
            pos++;
            actual++;
        }
        
        pos += actual;
        
        return actual;
    }
    
    public void reset() throws IOException
    {
        if(mark < 0) return;
        
        pos = mark;
        mark = -1;
    }
    
    public void reinit()
    {
        pos = 0;
    }
    
    public Object clone()
    { 
        return new BufferInputStream(buffer);
    }
    
    public long skip(long skip) throws IOException
    {
        skip = Math.min(buffer.length-pos, skip);
        pos += (int)skip;
        
        return skip;
    }
}

